package com.xhkjedu.smarking.service.report.report;

import com.xhkjedu.smarking.mapper.exam.MsClassMapper;
import com.xhkjedu.smarking.mapper.report.reportclass.MsrClassQuestionMapper;
import com.xhkjedu.smarking.mapper.report.reportclass.MsrClassSubjectGradeMapper;
import com.xhkjedu.smarking.mapper.report.reportother.MsrReportparamMapper;
import com.xhkjedu.smarking.mapper.report.reportsubject.MsrSubjectMapper;
import com.xhkjedu.smarking.model.exam.MsClass;
import com.xhkjedu.smarking.model.report.reportclass.MsrClassQuestion;
import com.xhkjedu.smarking.model.report.reportclass.MsrClassSubjectGrade;
import com.xhkjedu.smarking.model.report.reportother.MsrReportparam;
import com.xhkjedu.smarking.vo.report.reportother.ExamReportParams;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description：试题表
 * @Author: WN
 * @Date: 2024/12/24 9:47:39
 **/
@Service
public class MsrClassQuestionService {
    @Resource
    private MsrClassSubjectGradeMapper msrClassSubjectGradeMapper;
    @Resource
    private MsrSubjectMapper msrSubjectMapper;
    @Resource
    private MsrReportparamMapper msrReportparamMapper;
    @Resource
    private MsrClassQuestionMapper msrClassQuestionMapper;
    @Resource
    private MsClassMapper msClassMapper;


    //综合报告-试题汇总表-全部班级-科目情况
    public Map<String,Object> getSubjectForAllSmall(MsrClassSubjectGrade subjectGrade){
        //获取科目基础信息
        Map<String,Object> subjectMap = msrSubjectMapper.getSubjectBaseInfo(subjectGrade.getExamid(),subjectGrade.getSubjectid());
        subjectGrade.setClassid(0);
        subjectGrade.setReportcode("zh");
        //获取科目四率
        List<MsrClassSubjectGrade> subjectGradeList = msrClassSubjectGradeMapper.listSubjectGrade(subjectGrade);
        subjectMap.put("subjectGradeList",subjectGradeList);

        //获取四率参数
        MsrReportparam reportparam = msrReportparamMapper.getReportparamByRpbelongSubject(subjectGrade.getExamid(),"scorerate","zh",subjectGrade.getSubjectid());

        Map<String,Object> rtnMap = new HashMap<>();
        rtnMap.put("reportparam",reportparam);
        rtnMap.put("subject",subjectMap);

        return rtnMap;
    }
    //综合报告-试题汇总表-全部分类-按小题分析
    public Map<String,Object> listQuestionForAllSmall(ExamReportParams params){
        List<Map<String,Object>> questionList = msrClassQuestionMapper.listQuestionSummaryForSmall(params);
        List<Map<String,Object>> objlist;
        List<Map<String,Object>> sublist;
        if(params.getSelclass() == 2){//选择班级类型1全体 2全部班级对比 3指定班级对比 4单个班级
            //获取考试关联班级
            List<MsClass> classList= msClassMapper.listClassByExamidAndSubjectid(params.getExamid(),params.getSubjectid());

            objlist = new ArrayList<>();
            sublist = new ArrayList<>();

            //初始化学校全体
            List<Map<String,Object>> allSchool = questionList.stream().filter(m -> m.get("classid").equals(0)).collect(Collectors.toList());
            for(Map<String,Object> school : allSchool){
                school.put("classname","学校全体");
                if(school.get("qtype").equals("1")){
                    objlist.add(school);
                }else{
                    sublist.add(school);
                }
            }

            for(MsClass c : classList){
                List<Map<String,Object>> classQues = questionList.stream().filter(m -> m.get("classid").equals(c.getClassid())).collect(Collectors.toList());
                for(Map<String,Object> cq : classQues){
                    cq.put("classname",c.getClassname());
                    if(cq.get("qtype").equals("1")){
                        objlist.add(cq);
                    }else{
                        sublist.add(cq);
                    }
                }
            }
        }else{
            objlist = questionList.stream().filter(m -> m.get("qtype").equals("1")).collect(Collectors.toList());
            sublist = questionList.stream().filter(m -> m.get("qtype").equals("2")).collect(Collectors.toList());
        }

        Map<String,Object> rtnMap = new HashMap<>();
        rtnMap.put("objlist",objlist);
        rtnMap.put("sublist",sublist);
        return rtnMap;
    }
    //综合报告-试题汇总表-全部分类-按大题分析
    public List<Map<String,Object>> listQuestionForAllQtype(ExamReportParams params){
        return msrClassQuestionMapper.listQuestionSummaryForQtype(params);
    }

    //综合报告-试题汇总表-全部分类-按主(客)题分析
    public List<Map<String,Object>> listQuestionForAllObjsub(ExamReportParams params){
        return msrClassQuestionMapper.listQuestionSummaryForObjsub(params);
    }

    //学科报告-各班学科试题分析-得分率
    public void listQuestionForXhScoreRate(ExamReportParams params){
        params.setSelclass(3);
        params.setClassids("0," + params.getClassids());//拼上全体
        //获取选定班级的试题分析
        List<MsrClassQuestion> questions = msrClassQuestionMapper.listQuestionForXh(params);
        //获取所有题号
        List<String> qns = questions.stream().map(MsrClassQuestion::getQn).distinct().collect(Collectors.toList());
        //把试题按班级分组
        Map<Integer,List<MsrClassQuestion>> classMap = questions.stream().collect(Collectors.groupingBy(MsrClassQuestion::getClassid));
        List<Map<String,Object>> classList = new ArrayList<>();//班级试题
        for(Map.Entry<Integer,List<MsrClassQuestion>> entry : classMap.entrySet()){
            Map<String,Object> clMap = new HashMap<>();
            clMap.put("classid",entry.getKey());
            clMap.put("classname",entry.getValue().get(0).getClassname());
            clMap.put("questions",entry.getValue());
            classList.add(clMap);
        }
    }
    //学科报告-各班学科试题分析-试题难度和区分度
    public void listQuestionForXhNdAndQfd(ExamReportParams params){
        params.setSelclass(4);
        //获取选定班级的试题分析
        List<MsrClassQuestion> questions = msrClassQuestionMapper.listQuestionForXh(params);
        //获取所有题号
        List<String> qns = questions.stream().map(MsrClassQuestion::getQn).distinct().collect(Collectors.toList());
        //把试题按班级分组
        Map<Integer,List<MsrClassQuestion>> classMap = questions.stream().collect(Collectors.groupingBy(MsrClassQuestion::getClassid));
        List<Map<String,Object>> classList = new ArrayList<>();//班级试题
        for(Map.Entry<Integer,List<MsrClassQuestion>> entry : classMap.entrySet()){
            Map<String,Object> clMap = new HashMap<>();
            clMap.put("classid",entry.getKey());
            clMap.put("classname",entry.getValue().get(0).getClassname());
            clMap.put("questions",entry.getValue());
            classList.add(clMap);
        }
    }


}
